#### -- openBeads ----- plot line plots from protein arrays----------------------------
openBeads <- function (samples) {
# Function to load data from Honza's beads.R script. Unnecessary columns are discarded. Provided sample names ("samples") are added as column names.
  #
  # Args:
  #   samples: One of two vectors whose sample covariance is to be calculated.
  #   
  # Returns:
  #   List of variables. How to output multiple variables???

# load txt file which is output of apply_gate_template:
require( tcltk )
# choose file:
file_path <- tk_choose.files()
file_path <- file_path[2] # only last file (not default)

#filepdf <- paste(substr(file_path[2], 1, nchar(file_path[2])-4), ".pdf", sep="", collapse = NULL)
file_name <- tail(strsplit(file_path, "/")[[1]], n=1)

data_raw <- read.table(file_path, header = TRUE)
#data_raw <- t(read.csv(file_path, header = TRUE))

# numerical part of data.frame to new variable - data
data<- as.matrix(data_raw[,-c(1,2,3)]) # get rid of non-numerical columns
# use "antibody" column to name rows
rownames(data)<- data_raw[, which(colnames(data_raw)=="antibody")] 

# use "samples" for column names 
sample_nr <- ncol(data)/24   # number of samples (24 fractions repeats)
colnames_samples<-NULL # "initiate" colnames_samples??? why do I need it?
for (i in 1:sample_nr) { # number of samples
	for (k in 1:24) { # number of fractions
		colnames_samples[(i*24-23)+(k-1)] <- paste(samples[i], "_", k, sep = "") # "samples" contains sample names
	}
}
colnames(data) <- colnames_samples

# sort data based on row names:
data <- data[order(rownames(data)), ]
# data <- data[order(rownames(data)), order(colnames(data))]
# data <- data[ , order(colnames(data))]


# function output:
return(list(file_path=file_path, file_name=file_name, data_raw=data_raw, data=data, colnames_samples=colnames_samples))
}
#### -- END of openBeads ----------------------------------------------------------





#### -- backSub ----- normalisation of beads array data------------------------------
backSub <- function (data, samples, file_path, data_orig=data) {
# Function for background subtraction (based on "empty" beads)

frctns <- length(data[1,])/length(samples) # number of fractions per sample

# First find empty beads and make a data subset from them:
data_empty <- data[grep("empty", rownames(data)),]

# Prepare vectors of medians (, means and SDs) of empties:
mean_data_empty<-NULL # "iniciate" variable
median_data_empty<-NULL # "iniciate" variable
sd_data_empty<-NULL # "iniciate" variable
for (k in 1:ncol(data_empty)) {
	mean_data_empty<-append(mean_data_empty, mean(data_empty[,k]))
	median_data_empty<-append(median_data_empty, median(data_empty[,k]))
	sd_data_empty<-append(sd_data_empty, sd(data_empty[,k]))
}

# subtract median of empties from each row of data:
data_normalised_background<- t(t(data)-median_data_empty) 

# Save normalisation result - all lineplots:
linePlots(data, samples, file_path, data_name = "data", arbitrary_ylim = quantile(data, .99)) # original data
# prepare log data:
if (min(data)<0) {
	data_log <- log10(data+abs(min(data))+0.1)
} else {
	data_log <- log10(data+0.1)
}
linePlots(data_log, samples, file_path, data_name = "data_log", arbitrary_ylim = quantile(data_log, .99)) # original data log
linePlots(data_normalised_background, samples, file_path, data_name = "_norm_background", arbitrary_ylim = quantile(data_normalised_background, .99)) # back, no compare
linePlots(data_normalised_background, samples, file_path, data_name = "_norm_background_compare", compare_dataset = data, arbitrary_ylim = quantile(data, .99)) # back, compare to data

# Save normalisation result - data:
file_norm <- paste(substr(file_path, 1, nchar(file_path)-4), "_norm_background.txt", sep="", collapse = NULL)
# create header including column name ("antibody") for rawnames column
header<-c("antibody",colnames(data_normalised_background))
# save only header
write(header,file=file_norm, ncolumn=length(header),sep="\t")
# append data to the file 
write.table(data_normalised_background, file=file_norm, append=TRUE, sep="\t", col.names=FALSE)

# qc plot - does normalisation help?:
data_log<- log10(data+1.1)
x11()
par(mfrow=c(1,2))
boxplot(  rowSums(data_log[ , grep(samples[1], colnames(data))]), rowSums(data_log[ , grep(samples[2], colnames(data))]), rowSums(data_log[ , grep(samples[3], colnames(data))]), rowSums(data_log[ , grep(samples[4], colnames(data))]) , main="data_log" )
boxplot(  rowSums(data_normalised_background[ , grep(samples[1], colnames(data))]), rowSums(data_normalised_background[ , grep(samples[2], colnames(data))]), rowSums(data_normalised_background[ , grep(samples[3], colnames(data))]), rowSums(data_normalised_background[ , grep(samples[4], colnames(data))]), main="data_normalised_background"  )


return(list(file_path=file_path, data_normalised_background=data_normalised_background, data_empty=data_empty))
}
#### -- END of backSub -----------------------------------------------------------
















#### -- interNorm ---- Inter-sample normalisation-----------------------------------
interNorm <- function (data, samples, file_path, hskeepers=NULL, hskeepers_peaks=NULL) {
# Function for inter-sample normalisation. Based on centering of log data on average values in particular peaks in "housekeeping" proteins.

chosen_dataset <- data
## prepare log data:
#if (min(data)<0) {
#	chosen_dataset <- log10(data+abs(min(data))+0.1)
#} else {
#	chosen_dataset <- log10(data+0.1)
#}


# hskeepres (choice of "housekeeping" genes):
if (is.null(hskeepers)) {
  hskeepers<- "b2m_b2m" # which proteins
}

if (is.null(hskeepers_peaks)) {
  hskeepers_peaks <- as.matrix(t(c(19, 24)))
}
rownames(hskeepers_peaks) <- hskeepers
hskeepers_data <- NULL
for (i in 1:length(hskeepers)) {
	hskeepers_data <- rbind(hskeepers_data, chosen_dataset[grep(hskeepers[i], rownames(chosen_dataset), fixed=TRUE)[1],]) # all fractions (only first of grep)
}
# hskeepers mean:
frctns <- length(chosen_dataset[1,])/length(samples) # number of fractions per sample
temp_mean<-matrix(data=NA,nrow=length(hskeepers),ncol=length(samples))
temp_sd<-matrix(data=NA,nrow=length(hskeepers),ncol=length(samples))
for (i in 1:length(hskeepers)) {
	for (n in 1:length(samples)) { 
		# i-th hskeeper in n-th set of fractions:
		x<-hskeepers_data[i, (n*frctns-(frctns-1)):(n*frctns)]
		# mean of i-th hskeeper's peaks fractions ("average peak height") -> cbind all samples:
		temp_mean[i,n] <- mean(x[hskeepers_peaks[i,1]:hskeepers_peaks[i,2]])
		# sd of i-th hskeeper's peaks fractions -> cbind all samples:
		temp_sd[i,n] <- sd(x[hskeepers_peaks[i,1]:hskeepers_peaks[i,2]])
	}
}
hskeep_mean <- apply(temp_mean, 2, mean) # mean of hskeepers peaks' means, one/sample
hskeep_sd_mean <- apply(temp_sd, 2, mean) # mean of hskeepers peaks' sds, one/sample

# center data on hskeepers mean - log data:
data_internorm <- chosen_dataset
for (n in 1:length(samples)) { 
	data_internorm[, (n*frctns-(frctns-1)):(n*frctns)] <- scale(chosen_dataset[, (n*frctns-(frctns-1)):(n*frctns)], center = (rep(hskeep_mean[n], 24) - hskeep_mean[n]), scale = rep(hskeep_mean[n], 24)/hskeep_mean[1] ) # rep(hskeep_mean[n], 24)/hskeep_mean[1] scaling on first of house keepers mean (centering on average in each)
	#data_internorm[, (n*frctns-(frctns-1)):(n*frctns)] <- scale(chosen_dataset[, (n*frctns-(frctns-1)):(n*frctns)], center = rep(hskeep_mean[n], 24), scale = rep(hskeep_sd_mean[n], 24) )
}

# prepare log data:
if (min(data_internorm)<0) {
	data_internorm_log <- log10(data_internorm+abs(min(data_internorm))+0.1)
} else {
	data_internorm_log <- log10(data_internorm+0.1)
}

data_name = paste("internorm", paste(hskeepers, sep="", collapse="_"), sep="_", collapse="")

# line plots
#linePlots(data_internorm, samples, file_path, data_name = "internorm", compare_dataset = chosen_dataset, arbitrary_ylim = quantile(chosen_dataset, .99)) # with original data
linePlots(data_internorm, samples, file_path, data_name = paste("internorm", paste(hskeepers, sep="", collapse="_"), sep="_", collapse=""))
linePlots(data_internorm_log, samples, file_path, data_name = paste("internorm_log", paste(hskeepers, sep="", collapse="_"), sep="_", collapse=""), arbitrary_ylim = quantile(data_internorm_log, .79))  # log data

# Save normalisation result - data:
file_norm <- paste(substr(file_path, 1, nchar(file_path)-4), "_internorm", paste(hskeepers, sep="", collapse="_"), ".txt", sep="", collapse = NULL)
# create header including column name ("antibody") for rawnames column
header<-c("antibody",colnames(data_internorm))
# save only header
write(header,file=file_norm, ncolumn=length(header),sep="\t")
# append data to the file 
write.table(data_internorm, file=file_norm, append=TRUE, sep="\t", col.names=FALSE)

return(list(file_path=file_path, hskeepers=hskeepers, hskeepers_peaks=hskeepers_peaks, data_internorm=data_internorm, data_name=data_name))
}
#### -- END of interNorm ---------------------------------------------------------------










#### -- linePlots ----- plot line plots from protein arrays----------------------------
linePlots <- function (chosen_dataset, samples, file_path, data_name = "test", compare_dataset = 0, arbitrary_ylim = 0) {

frctns <- length(chosen_dataset[1,])/length(samples) # number of fractions per sample

#colormap <- c("red", "orange", "green", "black")
if (!exists("colormap")) {
	if (length(samples)>4) {
		colormap<-NULL
		for (i in 1:length(samples)) colormap[i] <- "grey"
	} else {
		colormap <- c("darkgoldenrod1", "orangered", "red3", "black")
	}
}

filepdf <- paste(substr(file_path, 1, nchar(file_path)-4), "_", data_name, ".pdf", sep="", collapse = NULL)
file_name <- tail(strsplit(filepdf, "/")[[1]], n=1)
pdf(filepdf,paper="a4r",width=0,height=0, title=paste(file_name)) # print to pdf (path is in filepdf)
for (i in 1:(ceiling(nrow(eval(chosen_dataset))/30)+1)){ # from 1 to round-up of # of rows in data divided by number of plots per graph (30) + 1 for legend
	par(mfrow=c(5,6), mai=c(0.3,0.3,0.2,0.02), cex.main=0.9, cex.axis=0.80, mgp = c(0, 0.5, 0)) # 6x5 plots in a graph (cex is text size, mai are margins, mgp fro axis labels distance)
	# First single plot of legend:
	if (i == 1) {
		plot(1, type="n", main="Legend", ylim=c(-(length(samples)+5), -0.5), xlim=c(0,10), axes=F, xlab="", ylab="")
		for (m in (length(samples):1)) {
			lines(c(0.5,2.5), c(-m,-m), type="l", lwd=4, col=colormap[m])
			text(3, -m, samples[m], adj=c(0,0), cex=0.8)
		}
		text(0.5, -(length(samples)+3), paste("Array: ", "..."), adj=c(0,0), cex=0.8)
		text(0.5, -(length(samples)+4), paste("file: ", file_name), adj=c(0,0), cex=0.8)
		plts_p_pg <- 29
	} else {
		plts_p_pg <- 30
	}
	# Then data plots:
	# (i*plts_p_pg-(plts_p_pg-1)):(i*plts_p_pg)
	for (k in (i*plts_p_pg-(plts_p_pg-(30-plts_p_pg))):(i*plts_p_pg+(29-plts_p_pg))){ # plot 30 plots
		if (k <= nrow(chosen_dataset)) {
			row_number<-k
			if (compare_dataset!=0) { # if there is original/comparison dataset
				#plot(chosen_dataset[row_number, grep(samples[1], colnames(chosen_dataset))], type="l", main=rownames(chosen_dataset)[row_number], xlab="", ylab="", ylim=c(min(c(chosen_dataset[k,], min(compare_dataset[k,]))),max(c(chosen_dataset[k,], max(compare_dataset[k,])))*1.01), col=colormap[1])
				#lines(compare_dataset[row_number, grep(samples[1], colnames(compare_dataset))], type="l", col=colormap[1], lty="dotted") # original/compare data are dotted
				plot(chosen_dataset[row_number, (1*frctns-(frctns-1)):(1*frctns)], type="l", main=rownames(chosen_dataset)[row_number], xlab="", ylab="", ylim=c(min(c(chosen_dataset[k,], min(compare_dataset[k,]))),max(c(chosen_dataset[k,], max(compare_dataset[k,])))*1.01), col=colormap[1])
				lines(compare_dataset[row_number, (1*frctns-(frctns-1)):(1*frctns)], type="l", col=colormap[1], lty="dotted") # original/compare data are dotted
			} else {
#				plot(chosen_dataset[row_number, grep(samples[1], colnames(chosen_dataset))], type="l", main=rownames(chosen_dataset)[row_number], xlab="", ylab="", ylim=c(min(chosen_dataset[k,]), max(arbitrary_ylim, max(chosen_dataset[k,])*1.01)), col=colormap[1])
				plot(chosen_dataset[row_number, (1*frctns-(frctns-1)):(1*frctns)], type="l", main=rownames(chosen_dataset)[row_number], xlab="", ylab="", ylim=c(min(chosen_dataset[k,]), max(arbitrary_ylim, max(chosen_dataset[k,])*1.01)), col=colormap[1])
			}
			if (length(samples) > 1) { 
				for (n in 2:length(samples)) { # start form 2nd sample
#					lines(chosen_dataset[row_number, grep(samples[n], colnames(chosen_dataset))], type="l", col=colormap[n])
					lines(chosen_dataset[row_number, (n*frctns-(frctns-1)):(n*frctns)], type="l", col=colormap[n])
					if (compare_dataset!=0) {  # if there is original/comparison dataset
#						lines(compare_dataset[row_number, grep(samples[n], colnames(compare_dataset))], type="l", col=colormap[n], lty="dotted") # original/compare data are dotted
						lines(compare_dataset[row_number, (n*frctns-(frctns-1)):(n*frctns)], type="l", col=colormap[n], lty="dotted") # original/compare data are dotted
					}
				}
			}
		axis(1,at= seq(from=1,to=24,by=1),labels=F, las=2, tck=-.02)
		abline(v=1:24,lwd=0.2,col="lightgrey") # vertical grid-lines
		abline(h=axTicks(side=2),lwd=0.2,col="lightgrey") # horizontal grid-lines
		}
	}
}
dev.off() # closing pdf

} # end of function
#### -- END of linePlots ----------------------------------------------------------



#### -- addBeads ---------------------------------------------------------------
addBeads <- function (add_to = data_normalised, add_data = "norm", samplenames = NULL) {
# Function to load data and merge them to existing data (either "raw" (3 non-data columns) or "norm" (1 non-data columns))
  #
  # Args:
  #		add_to:
  #		add_data:
  #		samplenames:
  #   
  # Returns:
  #   Merged dataset of add_to and interactively selected file

	require( tcltk ) # for shoosing file interactively
	if (add_data == "raw") {
		#samples <- samplenames
		x<-openBeads(samplenames) # function in "beads_functions.R" file
		cbind(add_to, x$data) # last => will be returned by function?
	} else if (add_data == "norm") {
		file_path <- tk_choose.files()
		file_path <- file_path[2] # only last file (not default)
		data <- read.table(file_path, header = TRUE, encoding = "UTF-8", check.names=FALSE) # read normalised data
		data_n<- as.matrix(data[,-1]) # get rid of non-numerical columns
		rownames(data_n) <- data[, 1] # first col has rownames
		cbind(add_to, data_n) # last => will be returned by function
	}
}
#### -- END of addBeads -------------------------------------------------------






#### -- openBeadsDir -------------------------------------------------------
openBeadsDir <- function (exp_path=NULL) {
# Function to load beads data (medians and quality control) from directory structure and put them into a data frame.
  #
  # Args:
  #		exp_path: optional, in case path is added manualy (no trailing / )
  #   
  # Returns:
  #   data frame containing all info in named columns

	require( tcltk ) # for shoosing file interactively
	require( ggplot2 )
	require( reshape )

	# exp_path (where the experiment files are):
	if (is.null(exp_path)) {
		exp_path <- tk_choose.dir()
	}
	# list of files with sub-paths:
	exp_files <- list.files(path = exp_path, recursive = TRUE)
	# variables of absolute paths to experiment files:
	qc_path <- paste(exp_path, "/", exp_files[grep("qc.txt", exp_files)], sep="")
	mininfo_path <- paste(exp_path, "/", exp_files[grep("MININFO", exp_files)], sep="")
	medians_desp_path <- paste(exp_path, "/", exp_files[grep("MEDIANS_DESP", exp_files)], sep="")
	medians_path <- paste(exp_path, "/", exp_files[grep("MEDIANS", exp_files)][-grep("MEDIANS_DESP", exp_files[grep("MEDIANS", exp_files)])], sep="")
	frame_path <- paste(exp_path, "/", exp_files[grep("frame", exp_files)], sep="")
	file_name <- strsplit(qc_path[1], "QC/")[[1]][2] # get just file_name
	file_name <- substr(file_name, 1, nchar(file_name)-7)
	
	# load data into temporary variables:
	# 0)
	# load just first column of medians to store row labels (Ab_IDs):
	beads_rownames <- read.table(paste(exp_path, "/", exp_files[grep("MEDIANS", exp_files)][-grep("MEDIANS_DESP", exp_files[grep("MEDIANS", exp_files)])], sep=""), header = TRUE, stringsAsFactors = FALSE, encoding = "UTF-8", check.names=FALSE)[, 1]
	# load first row, exluding first 3 columns to get columnames for samples extraction
	beads_colnames <- read.table(paste(exp_path, "/", exp_files[grep("MEDIANS", exp_files)][-grep("MEDIANS_DESP", exp_files[grep("MEDIANS", exp_files)])], sep=""), stringsAsFactors = FALSE, encoding = "UTF-8", check.names=FALSE)[1, -c(1,2,3)]
	# samples:
	samples <- unique(sub(pattern="_[0-9]+$", x=beads_colnames, replacement="") )
	# 1) medians:
	medians <- as.matrix(read.table(paste(exp_path, "/", exp_files[grep("MEDIANS", exp_files)][-grep("MEDIANS_DESP", exp_files[grep("MEDIANS", exp_files)])], sep=""), header = TRUE, stringsAsFactors = FALSE, encoding = "UTF-8", check.names=FALSE)[ , -c(1,2,3)]) # don't load non-numerical columns, load as (numerical) matrix
	# 2) medians despikified
	medians_desp <- as.matrix(read.table(paste(exp_path, "/", exp_files[grep("MEDIANS_DESP", exp_files)], sep=""), header = TRUE, stringsAsFactors = FALSE, encoding = "UTF-8", check.names=FALSE)[ , -c(1,2,3)])
	# 3) minimal (bead array experimnt) info:
	mininfo <- read.table(paste(exp_path, "/", exp_files[grep("MININFO", exp_files)], sep=""), sep="", encoding = "UTF-8", check.names=FALSE)
	###exp_desc <- as.matrix(mininfo[ , 4])
	###rownames(exp_desc) <- mininfo[ , 3]
	# 4) quality control file (variables: qc+popsize, qc_modality)
	qc_popsize <- as.matrix(read.table(paste(exp_path, "/", exp_files[grep("qc.txt", exp_files)], sep=""), header=TRUE, stringsAsFactors=FALSE, encoding = "UTF-8", check.names=FALSE)[1:length(beads_rownames),-1])
	
	qc_modality <- as.matrix(read.table(paste(exp_path, "/", exp_files[grep("qc.txt", exp_files)], sep=""), header=TRUE, stringsAsFactors=FALSE, encoding = "UTF-8", check.names=FALSE)[(length(beads_rownames)+1):(2*length(beads_rownames)),-1])
	###apply(qc_popsize, MARGIN = 2, FUN = min)

	# beads_melted variable (medians + qc):
	fractions<-NULL
	for (i in 1:24) fractions <- c(fractions, rep(i, length(beads_rownames)))

	beads_melted <- cbind(
	data.frame(Fractions = rep(fractions, length(samples))),
	data.frame(Array_row = rep(1:length(beads_rownames), length(beads_colnames))),
	data.frame(Ab_ID = rep(beads_rownames, length(beads_colnames))),
	data.frame(Columns = melt(qc_popsize)[, 2]),
	data.frame(Popsize = melt(qc_popsize)[, 3]),
	data.frame(Samples = sub("_..$","", sub("_.$","", data.frame(melt(qc_popsize))[, 2]) )),
	data.frame(Modality = melt(qc_modality)$value),
	data.frame(Medians = melt(medians)$value),
	data.frame(Medians_desp = melt(medians_desp)$value)
	 )
	
	# prepare meadians_desp as "data_desp" with rownames, colnames for further analysis (remove when casting of beads_melted works)
	data_desp<-medians_desp
	rownames(data_desp) <- beads_melted$Ab_ID[1:max(beads_melted$Array_row)]
	colnames(data_desp) <- beads_colnames
	data_desp <- data_desp[order(rownames(data_desp)), ] # sort data based on row names
	
#return(beads_melted)
return(list(beads_melted=beads_melted, medians_desp=medians_desp, exp_path=exp_path, file_name=file_name, mininfo=mininfo, beads_colnames=beads_colnames, samples=samples, data_desp=data_desp))
}
#### -- END of openBeadsDir -------------------------------------------------------





#### -- qcReport -------------------------------------------------------
qcReport <- function (beads_melted, exp_path, mininfo=NULL, colormap=NULL) {
# Function to load beads data (medians and quality control) from directory structure and put them into a data frame.
  #
  # Args:
  #		beads_melted: output of openBeadsDir function (hold data of medians, medians_desp, popsize, modality
  #		exp_path: optional, in case path is added manualy (no trailing / )
  #		colormap: colors of samples
  #		mininfo: information about experiment
  #   
  # Returns:
  #   no return, just pdf file

	# prepare file_name:
	exp_files <- list.files(path = exp_path, recursive = TRUE)
	# variables of absolute paths to experiment files:
	qc_path <- paste(exp_path, "/", exp_files[grep("qc.txt", exp_files)], sep="")
	file_name <- strsplit(qc_path[1], "QC/")[[1]][2] # get just file_name
	file_name <- substr(file_name, 1, nchar(file_name)-7)

	samples <- as.character(levels(beads_melted$Samples))

	# qc pdf report:	
	filepdf <- paste(exp_path, "/QC/", file_name, "_qc_report.pdf", sep="", collapse = NULL)
	file_name_qc <- tail(strsplit(filepdf, "/")[[1]], n=1)
	pdf(filepdf,paper="a4r",width=0,height=0, title=paste(file_name_qc)) # print to pdf (path is in filepdf)
	
	#colormap
	if (is.null(colormap)) { # if colormat is NULL, create colormap
		if (length(samples)>4) {
			colormap<-NULL
			for (i in 1:length(samples)) colormap[i] <- "grey"
		} else {
			colormap <- c("darkgoldenrod1", "orangered", "red3", "black")
		}
	}
	# mininfo text:
	if (!is.null(mininfo)){ # if there is mininfo do:
        cex_text<-matrix(0.7,nrow(mininfo),ncol(mininfo))
        plot(0,xlim=c(0, 8), type="n", ylim=c(0,30), axes=FALSE, frame.plot=FALSE, ann=FALSE)
        for (i in 1:length(mininfo[, 1])){
            for(j in 1:length(mininfo[1, ])){
                text(j*2.2-1.5,30-i,mininfo[i,j],cex=cex_text[i,j])
            }
        }
    }
	# plots:
	# Popsize:
	# Needs xlim, otherwise will order fractions strangely
	 print(ggplot(data=beads_melted, aes(Columns, Popsize)) + geom_boxplot() + xlim(as.character(unique(beads_melted$Columns))) + opts(title = expression("Number of beads (popsize) in FPLC fractions"))  )

	# number of beads (popsize) distribution acros fractions and samples
	print(ggplot(data=beads_melted, aes(factor(Fractions), Popsize)) + geom_boxplot(aes(fill = factor(Samples))) + scale_fill_manual(values = colormap) + opts(title = expression("Number of beads (popsize) in FPLC fractions"))  )
	
	# Modality in PE detector distribution across fractions and samples
	print(ggplot(data=beads_melted, aes(factor(Fractions), Modality)) + geom_jitter(aes(colour = factor(Samples), alpha = 0.2)) + scale_colour_manual(values = colormap) + opts(title = expression("Modality in PE detector distribution across fractions and samples"))  )

	# Medians (of PE detector) distribution across fractions and samples 
#	print(ggplot(data=beads_melted, aes(factor(Fractions), Medians)) + geom_jitter(aes(colour = factor(Samples), alpha = 0.2)) + scale_colour_manual(values = colormap) + scale_y_log10())
#	print(ggplot(data=beads_melted, aes(factor(Fractions), Medians_desp)) + geom_jitter(aes(colour = factor(Samples), alpha = 0.2)) + scale_colour_manual(values = colormap) + scale_y_log10())	
	print(ggplot(data=beads_melted, aes(factor(Fractions), Medians)) + geom_boxplot(aes(fill = factor(Samples))) + scale_fill_manual(values = colormap) + scale_y_log10() + opts(title = expression("Medians (of PE detector) distribution across fractions and samples"))  )
	print(ggplot(data=beads_melted, aes(factor(Fractions), Medians_desp)) + geom_boxplot(aes(fill = factor(Samples))) + scale_fill_manual(values = colormap) + scale_y_log10() + opts(title = expression("Medians-despikified (of PE detector) distribution across fractions and samples"))  ) 
	print(ggplot(data=beads_melted, aes(factor(Samples), Medians_desp)) + geom_boxplot(aes(fill = factor(Samples))) + scale_fill_manual(values = colormap) + scale_y_log10() + opts(title = expression("Medians-despikified (of PE detector) distribution in samples"))  )
	 

	dev.off() # closing pdf

}
#### -- END of qcReport -------------------------------------------------------






#### -- beadsPeaksManualy -------------------------------------------------------
beadsPeaksManualy <- function (data=NULL, file_path=NULL, file_name=NULL, samples=NULL, array_list=NULL, peaks_list_path="test_peaks", data_name=NULL) {
# Peak detection based on manual peak annotation. Asuming we have available:
#	data (eg. TOM1_K562_all_norm.txt )
#	list of peaks (eg. ProtArrPeaks_list_Karel2.txt )
#	array description file (eg. array65_58_list.csv )

  #
  # Args:
  #		data: normalised data
  #		file_path
  #		file_name
  #		peaks_list_path: path to peaks list (to read peaks list and to save info on which list (file name) it was)
  #		array_list
  #		samples
  #		data_name: if data passed from internorm, data_name contains hskeepers names
  #   
  # Returns:
  #   data frame containing all info in named columns


# load txt file which is output of apply_gate_template:
require( tcltk )
# choose file (normalised data):
if (!exists("file_path")) {
	file_path <- tk_choose.files()
	file_path <- file_path[2] # only last file (not default)
}

if (is.null(file_name)) {
	file_name <- tail(strsplit(file_path, "/")[[1]], n=1)
}




# Reconstruct sample names:
if (!exists("samples")) {
	samples <- NULL
	for (i in 1:(length(colnames(data))/24)) {
		samples <- c(samples, substr(colnames(data)[i*24-23], 1, nchar(colnames(data)[i*24-23])-2))
	}
}

# Number of fractions per sample:
frctns <- length(data[1,])/length(samples) # number of fractions per sample

# read list of annotated peaks from file:
if (!exists("peaks_list_path")) {
	peaks_list <- read.table(file="/home/karel/DATA_Lab/Protein_arrays/ProtArrPeaks/ProtArrPeaks_lists/ProtArrPeaks_list_VKKF3_ID.txt", header=TRUE)
} else {
peaks_list <- read.table(file=peaks_list_path, header=TRUE, sep="\t")
}
peaks_file_name <- tail(strsplit(peaks_list_path, "/")[[1]], n=1)
peaks_list <- peaks_list[order(peaks_list$ID), ]
IDs <- peaks_list$ID # list of IDs


# file_peak is path including file name of processed peaks data (and pdf)
if (!is.null("data_name")) { # if data_name supplied as parameter of a function
	file_peak <- paste(substr(file_path, 1, nchar(file_path)-4), "_", data_name, "_", substr(peaks_file_name, 1, nchar(peaks_file_name)-4), ".txt", sep="", collapse = NULL)
} else {
	file_peak <- paste(substr(file_path, 1, nchar(file_path)-4), substr(peaks_file_name, 1, nchar(peaks_file_name)-4), ".txt", sep="", collapse = NULL)
}


# read array info from file:
if (!exists("array_list")) {
	array_list <- read.csv(file="/home/karel/DATA_Lab/Protein_arrays/ProtArr_lists/array65_58_list.csv", header=TRUE)
}
array_list$ID

# prepare list of uniprots (mathcing IDs from peaks_list to array info)
uniprot <- NULL
for (i in (1:length(IDs))) {
	uniprot[i] <- as.character(array_list[match(IDs[i], array_list$ID), colnames(array_list)=="uniprot"])
}

# prepare list of HGNC symbols (names) (mathcing IDs from peaks_list to array info)
name <- NULL
for (i in (1:length(IDs))) {
	name[i] <- as.character(array_list[match(IDs[i], array_list$ID), colnames(array_list)=="name"])
}

# prepare list of weird identifiers (antibody) (mathcing IDs from peaks_list to array info)
antibody <- NULL
for (i in (1:length(IDs))) {
	antibody[i] <- as.character(array_list[match(IDs[i], array_list$ID), colnames(array_list)=="antibody"])
}




# New rownames for data --REMOVE WHEN PROPER NAMES AS STANDARD---:
rownames(data) <- array_list$ID[order(array_list$antibody)] # asign rownames based on IDs in array_list but ordered by "antibody" field
#--REMOVE WHEN PROPER NAMES AS STANDARD---


# Make data subset based on provided peaks list:
search_proteins <- NULL
data_search <- NULL
for (i in 1:length(IDs)) {
	search_proteins <- as.character(IDs[i])
	data_search <- rbind(data_search, data[grep(search_proteins, rownames(data))[1],]) # looking for (==grep) "search_proteins" in rownames of "data" and use only first match to return the data from data for this search and rbind it to "data_search".
	rownames(data_search)[i] <- rownames(data)[grep(search_proteins, rownames(data))[1]]
}

# Make set of peaks sums ("area under the peak curves):
peaks_data <- NULL; peaks_data <- as.data.frame(peaks_data)
peak_count <- 0
peaks_info <- NULL
peaks_uniprot <- NULL # for hyperlinks in svg need uniprot for each peak
peaks_name <- NULL
peaks_antibody <- NULL
for (prot_nr in 1:length(peaks_list$ID)) { # cycle throught annotated proteins 
	x <- peaks_list[prot_nr,which(is.na(peaks_list[prot_nr, 4:length(peaks_list)])!=TRUE)+3]
	protein_uniprot <- as.character(array_list[match(IDs[prot_nr], array_list$ID), colnames(array_list)=="uniprot"]) # uniprot for protein
	protein_name <- as.character(array_list[match(IDs[prot_nr], array_list$ID), colnames(array_list)=="name"])
	protein_antibody <- as.character(array_list[match(IDs[prot_nr], array_list$ID), colnames(array_list)=="antibody"]) # remove this! old "antibody" name!!!
	for (peak_nr in 1:(length(x)/3)) { # cycle throught peaks in a protein
		x_s <- as.numeric(x[grep("_s", colnames(x))[peak_nr]]) # peak start ("_s")
		x_h <- as.numeric(x[grep("_h", colnames(x))[peak_nr]]) # peak height ("_h")
		x_e <- as.numeric(x[grep("_e", colnames(x))[peak_nr]]) # peak end ("_e")
		print(paste(as.character(array_list[match(IDs[prot_nr], array_list$ID), colnames(array_list)=="ID"]), ": prot_nr-", prot_nr, "peak-", peak_nr, ": ", x_s, x_h, x_e))
		peak_count <- peak_count+1 # add peak number to peak counter
		y <- NULL # reset y for peaks_info		
		for (sample in 1:length(samples)) { # cycle throught samples in a peak in a protein
			#test data from data_search, only one sample and one protein:
			test_data <- data_search[prot_nr,(sample*24-23):(sample*24)]
			peak_sum <- sum(test_data[x_s:x_e])
			peaks_data[peak_count, sample] <- peak_sum
			rownames(peaks_data)[peak_count] <- paste(rownames(data_search)[prot_nr], "_Pk", peak_nr, "_sum", sep="")						
			y <- c(x_s, x_h, x_e) # vector		
		}
		y <- c(y, prot_nr)
		peaks_info <- rbind(peaks_info, y)
		peaks_uniprot <- c(peaks_uniprot, protein_uniprot)
		peaks_name <- c(peaks_name, protein_name)
		peaks_antibody <- c(peaks_antibody, protein_antibody)
	}	
}
colnames(peaks_data) <- samples
peaks_info <- as.data.frame(peaks_info, row.names = rownames(peaks_data))
colnames(peaks_info) <- c("x_s", "x_h", "x_e", "prot_nr")
peaks_data <- peaks_data[order(rownames(peaks_data)), ]
peaks_info <- peaks_info[order(rownames(peaks_info)), ]


# Save peaks detection result - all lineplots:
if (!exists("colormap")) {
	if (length(samples)>4) {
		colormap<-NULL
		for (i in 1:length(samples)) colormap[i] <- "grey"
	} else {
		colormap <- c("darkgoldenrod1", "orangered", "red3", "black")
	}
}

# plot peaks into pdf:
filepdf <- paste(substr(file_peak, 1, nchar(file_peak)-4), ".pdf", sep="", collapse = NULL)
pdf(filepdf, paper="a4r",width=0, height=0, title=paste(file_name, " - ", peaks_file_name)) # print to pdf (path is in filepdf)
for (i in 1:ceiling(nrow(peaks_info)/30)){ # from 1 to round-up of # of rows in data divided by number of plots per graph (30)
	par(mfrow=c(5,6), mai=c(0.3,0.3,0.2,0.02), cex.main=1, cex.axis=0.80, mgp = c(0, 0.5, 0)) # 6x5 plots in a graph (cex is text size, mai are margins, mgp fro axis labels distance)
	# First single plot of legend:
	if (i == 1) {
		plot(1, type="n", main="Legend", ylim=c(-(length(samples)+5), -0.5), xlim=c(0,10), axes=F, xlab="", ylab="")
		for (m in (length(samples):1)) {
			lines(c(0.5,2.5), c(-m,-m), type="l", lwd=4, col=colormap[m])
			text(3, -m, samples[m], adj=c(0,0), cex=0.8)
		}
		text(0.5, -(length(samples)+3), paste("Array: ", "..."), adj=c(0,0), cex=0.8)
		text(0.5, -(length(samples)+4), paste("file: ", file_name), adj=c(0,0), cex=0.8)
		plts_p_pg <- 29
	} else {
		plts_p_pg <- 30
	}	
	# Then data plots:
	# plots per page (plts_p_pg)
	for (k in (i*plts_p_pg-(plts_p_pg-1)):(i*plts_p_pg)){ # plot 30 plots
		if (k <= nrow(peaks_info)) {
			row_number<-k
			plot(data_search[peaks_info[row_number,4], (1*frctns-(frctns-1)):(1*frctns)], type="l", main=substr(rownames(peaks_data)[row_number], 1, nchar(rownames(peaks_data)[row_number])-4), xlab="", ylab="", ylim=c(min(data_search[peaks_info[row_number,4],]),max(c(1.5, max(data_search[peaks_info[row_number,4],])*1.01))), col=colormap[1]) # title not displaying "_sum" part of peaks rownames
			# ylim=c(0,max(data_search[peaks_info[row_number,4],])*1.01)
			abline(v=1:24,lwd=0.2,col="lightgrey")
			abline(h=axTicks(side=2),lwd=0.2,col="lightgrey")
			# ablines are over data, so need to plot same data again! :-(
			par(new = TRUE)
			plot(data_search[peaks_info[row_number,4], (1*frctns-(frctns-1)):(1*frctns)], type="l", main=substr(rownames(peaks_data)[row_number], 1, nchar(rownames(peaks_data)[row_number])-4), xlab="", ylab="", ylim=c(min(data_search[peaks_info[row_number,4],]),max(c(1.5, max(data_search[peaks_info[row_number,4],])*1.01))), col=colormap[1]) # title not displaying "_sum" part of peaks rownames
			lines(peaks_info[row_number,1]:peaks_info[row_number,3], data_search[peaks_info[row_number,4], peaks_info[row_number,1]:peaks_info[row_number,3]], col=colormap[1], type="p", pch=20)
			abline(v=peaks_info[row_number,2], col=4, lty="dotted")
			mtext(paste("http://www.uniprot.org/uniprot/", peaks_uniprot[row_number], sep=""), cex = 0.5, line = 0.5, side = 3, col = rgb(1, 1, 1, 0.1)) # cex => big, side 3 is top, col has 4th value=alpha
# REMOVE!	# add old "antibody" name --> GRRRRRRRRRRRRRRRRRRRRRRr:
			mtext(peaks_antibody[row_number], cex = 0.5, line = -0.8, side = 3)
			if (length(samples) > 1) {			
				for (n in 2:length(samples)) {
					lines(data_search[peaks_info[row_number,4], (n*frctns-(frctns-1)):(n*frctns)], type="l", col=colormap[n])
					lines(peaks_info[row_number,1]:peaks_info[row_number,3], data_search[peaks_info[row_number,4], (peaks_info[row_number,1]+24*(n-1)):(peaks_info[row_number,3]+24*(n-1))], col=colormap[n], type="p", pch=20)
				}
			}
		axis(1,at= seq(from=1,to=24,by=1),labels=F, las=2, tck=-.02)
		}
	}
}

dev.off() # closing pdf


# Save peaks detection result - data:

# create header including column name ("antibody") for rawnames column
header<-c("antibody_peak_sum", colnames(peaks_data))
# save only header
write(header,file=file_peak, ncolumn=length(header),sep="\t")
# append data to the file 
write.table(peaks_data, file=file_peak, append=TRUE, sep="\t", col.names=FALSE)



return(list(file_peak=file_peak, file_path=file_path, file_name=file_name, samples=samples, uniprot=uniprot, peaks_data=peaks_data, peaks_info=peaks_info, peaks_uniprot=peaks_uniprot, peaks_name=peaks_name)  )
}
#### -- END of beadsPeaksManualy -------------------------------------------------------



#### -- analyseBeads ------------wrapper function----------------------------------------
analyseBeads <- function (peaks_list_path, array_list, hskeepers, hskeepers_peaks, exp_path=NULL, peaks_on_data="internorm") {
  # Wrapper function to proccess beads data: load data form directory structure,
  # subtract backgrounf, internormalise on chosen housekeeping proteins, 
  # calculate sums of peaks signals (defined manuali in "Peaks list").
  
  
  require( tcltk ) # for shoosing file interactively
  require( ggplot2 )
  require( reshape )
  
  # exp_path (where the experiment files are):
  if (is.null(exp_path)) {
    exp_path <- tk_choose.dir()
  }
  
  #----- load directory and save to variables
  x1 <- openBeadsDir(exp_path=exp_path)
  for (i in names(x1)) { assign(i, x1[[i]]) }
  #----- Create QC Report pdf
  qcReport(beads_melted=beads_melted, exp_path=exp_path, mininfo=mininfo)
  # -- intra-normalisation (backround correction):
  x1 <- backSub(data_desp, samples, file_path=paste(exp_path, "/ANALYSIS/", file_name, ".txt", sep="", collapse = NULL), data_orig=data_desp)
  for (i in names(x1)) { assign(i, x1[[i]]) }
  # -- inter-normalisation:
  x1 <- interNorm(data_normalised_background, samples=samples, file_path=file_path, hskeepers=hskeepers, hskeepers_peaks=hskeepers_peaks)
  for (i in names(x1)) { assign(i, x1[[i]]) }
  # -- Peak detection:
  if (peaks_on_data=="background") {
    data_normalised <- data_normalised_background
    data_name = "background"
  } else {
    data_normalised <- data_internorm
  }
  x1 <- beadsPeaksManualy(data=data_normalised, file_path=file_path, data_name=data_name,  samples=samples,  peaks_list_path=peaks_list_path, array_list=array_list)
  for (i in names(x1)) { assign(i, x1[[i]]) }
  
  # save workspace
  save.image(file = paste(substr(file_peak, 1, nchar(file_path)-4), ".RData", sep="", collapse = NULL))
  
  return(list(beads_melted=beads_melted, medians_desp=medians_desp, exp_path=exp_path, file_name=file_name, mininfo=mininfo, beads_colnames=beads_colnames, samples=samples, data_desp=data_desp, file_path=file_path, data_normalised_background=data_normalised_background, data_empty=data_empty, hskeepers=hskeepers, hskeepers_peaks=hskeepers_peaks, data_internorm=data_internorm, data_name=data_name, uniprot=uniprot, peaks_data=peaks_data, peaks_info=peaks_info, peaks_uniprot=peaks_uniprot, peaks_name=peaks_name))
}
#### -- END of analyseBeads ---------------------------------------------------------






