\name{analyseBeads}
\alias{analyseBeads}
\title{Background substraction, intersample normalization and entities
  extraction}
\description{
 Transform the samples and extract protein entities

}
\usage{
        analyseBeads(peaks_list_path, array_list,hskeepers, hskeepers_peaks)

          }
\arguments{
  \item{peaks_list_path}{character string providing the path to the text
  file with peaks (entities) definition}
  \item{array_list}{character string providing the path to the csv file
    of array desrciption}
  \item{hskeepers}{vector of character strings - the names of housekeeping proteins used for normalization}
  \item{hskeepers_peaks}{integer matrix with nrow(hskeepers_peaks)==length(hskeepers), first column the starting indices of protein entity on lineplot, second column the ending indices}
 
}

\details{Due to nonspecific binding of proteins directly on the surfaces of microspheres, we observe low levels of background noise, which also contribute to the variability of signals near background levels. To correct for this noise, the signal from empty microspheres (microspheres without any antibody) is subtracted from the signal of the remaining microsphere populations.

The acquired microsphere population PE-MFI values depend on the total amount of protein loaded in the sample. To correctly compare values across samples, a protein load correction is applied. This is based on proteins for which expression does not change under particular experimental conditions, i.e., housekeeping proteins.

The normalized data are then processed with respect to peaks definition}

\examples{
##to run this example you need to visit site
##www.bioinformin.net/sample.php a download data sample in decribed way.
library(rgbeads)
##Now load the prepared frame.
load("archframe")
## This frame is result of the function edit_structure() and contains only description of structure of the array.
## Now we add to this frame information about population distribution in the training file(s).
array6558<-create_gate_frame(frame=archframe,inputfile=c("1ST_C1_C01_LOW.fcs","1ST_C1_C01_HIGH.fcs"),popdesc="array6558.txt")
##Now we need to inspect this frame and add to it the spatial template for rapid batch gating.
array6558<-inspect_gate_frame(array6558)
##When you have checked all population, type ”finish” (enter), and wait a while, while template is being
##created.
##The frame is now completed, it contains all information (structure of array, missing population, spatial template)
##and we can save it!
save(array6558, file="./exp_source/array6558")
##And now eventually we can extract the medians!
##check if the directory ./exp_source exists and contains all necessary files
apply_gate_template(directory="./exp_source")

# source KF functions:
source("/path_to/beads_functions_kf.R")
# prepare variables::
peaks_list_path <- "ProtArrPeaks_list_6t.txt"
# path to peaks definition file
array_list <- read.csv(file="array65_58_list.csv", header=TRUE)
# array list with antibody IDs and uniprots
hskeepers<- c("b2m_b2m", "HES1_1F5", "VRK1_1F6") # which proteins
hskeepers_peaks <- rbind(c(19, 24), c(15, 21), c(14, 18))
# start and end of peaks
x <- analyseBeads(peaks_list_path=peaks_list_path, array_list=array_list,
hskeepers=hskeepers, hskeepers_peaks=hskeepers_peaks)
# run the function


        
 }
